﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using Rewired;


public class Weapon : MonoBehaviour {

    public float fireRate = 0;
    public int damage = 10;
    public LayerMask whatToHit;

    private ArrayList listWasteland;
    private ArrayList listStoneWasteland;



    public Transform spawnFartPosition;
    public Transform spawnWindPosition;
    public Transform BulletTrailPrefab;
    public Transform MuzzleFlashPrefab;
    public Transform HitPrefab;


    public float speedFart;

    float timeToSpawnEffect = 0;
    public float effectSpawnRate = 10;

    public float camShakeAmt = 0.05f;
    public float camShakeLenght = 0.1f;
    CameraShake camShake;

    float timeToFire = 0;
    Transform firePoint;

    public float nextFart;
    public float nextBullet;
    public float nextShot;

    bool canFart = true;

    bool canShot = true;

    bool isFarting = false;

    Animator animPlayer;
    Player player;
    PlayerMovement pm;

    public bool isDead = false;

    AudioSource source;
    public AudioClip windShoot;
    public AudioClip snot;
    public AudioClip superSnot;
    public AudioClip turbo;
    public AudioClip deadSound;

    void Awake()
    {
        source = GetComponent<AudioSource>();
        pm = GetComponent<PlayerMovement>();
        //DontDestroyOnLoad(gameObject);

        firePoint = transform.FindChild("FirePoint");
        if(firePoint == null)
        {
            firePoint = transform;
        }

        listWasteland = new ArrayList();
        listStoneWasteland = new ArrayList();

        foreach(Transform tile in GameObject.Find("Tiles").transform)
        {
            if(tile.name == "wasteland")
            {
                listWasteland.Add(tile);
                //print(tile.name);
            }

            if (tile.name.Contains("stone-wasteland"))
            {
                listStoneWasteland.Add(tile);
                //print(tile.name);
            }


        }
    }

    

    // Use this for initialization
    void Start () {

        player = GetComponent<PlayerMovement>().player;

        animPlayer = GetComponent<Animator>();

        nextBullet = 0f;
        nextShot = 1f;
        
        //camShake = GameMaster.gm.GetComponent<CameraShake>();

        /*
        if(camShake == null)
        {
            Debug.LogError("No CameraShake script");
        }*/
	}
	


    void CheckDeath()
    {
        foreach (Transform tile in listWasteland)
        {
            float dist = Vector2.Distance(tile.position, transform.position);
            if (dist < 0.5)
            {
                Death();
                
                return;
            }
        }
    }

    void ChangePlayerCamera()
    {
        Camera main = Camera.main;
        main.GetComponent<CameraMovement2>().targets = new Transform[1];
        main.GetComponent<CameraMovement2>().boundingBoxPadding = 2;
        main.GetComponent<CameraMovement2>().minimumOrthographicSize = 1.5f;
        main.GetComponent<CameraMovement2>().zoomSpeed = 5;
        if (GetComponent<PlayerMovement>().playerNumber == 0)
        {
            main.GetComponent<CameraMovement2>().targets[0] = GameObject.Find("Player_1").transform;
            GameObject.Find("P1Win").GetComponent<CanvasGroup>().alpha = 1;
            //

            GameMaster.gm.remainingLivesP2 = GameMaster.gm.remainingLivesP2 - 1;

            GameObject.Find("TextLife2").GetComponent<Text>().text = "X "+ GameMaster.gm.remainingLivesP2;

            

        }
        else
        {

            main.GetComponent<CameraMovement2>().targets[0] = GameObject.Find("Player_0").transform;
            GameObject.Find("P2Win").GetComponent<CanvasGroup>().alpha = 1;

            GameMaster.gm.remainingLivesP1 = GameMaster.gm.remainingLivesP1 - 1;
            GameObject.Find("TextLife1").GetComponent<Text>().text = "X " + GameMaster.gm.remainingLivesP1;
        }


        GameObject.Find("Life1").GetComponent<CanvasGroup>().alpha = 0;
        GameObject.Find("Life2").GetComponent<CanvasGroup>().alpha = 0;
    }


    void Death()
    {
        //print("MOOOOOORTOOOOO");
        isDead = true;


        if (GetComponent<PlayerMovement>().playerNumber == 0)
        {
            GameObject.Find("Player_1").GetComponent<Rigidbody2D>().drag = 100;
        }
        else
        {
            GameObject.Find("Player_0").GetComponent<Rigidbody2D>().drag = 100;
        }



        InvokeRepeating("AnimationDeath", 0f, 1/60f);
        GetComponent<Rigidbody2D>().drag = 100;

        source.PlayOneShot(deadSound);

    }

    void AnimationDeath()
    {
        if (transform.localScale.x > 0) {
            transform.localScale -= new Vector3(0.01f, 0.01f, transform.localScale.z);
        }
        else
        {
            CancelInvoke("AnimationDeath");
            ChangePlayerCamera();

            GameMaster.gm.WaitWinScreen();
        }
    }





    void CheckEdgeTile()
    {
        foreach (Transform tile in listStoneWasteland)
        {
            float dist = Vector2.Distance(tile.position, transform.position);
            if (dist < 0.5)
            {
                //print("SLOOOOOW");
                SlowDown();
                return;
            }
        }

        GetComponent<PlayerMovement>().speed = 8.0f;
    }

    void SlowDown()
    {
        GetComponent<PlayerMovement>().speed = 4f;
        //GetComponent<Rigidbody2D>().velocity = GetComponent<Rigidbody2D>().velocity * 0.3f;

        //GetComponent<Rigidbody2D>().angularVelocity = GetComponent<Rigidbody2D>().angularVelocity * 0.3f;
    }






	// Update is called once per frame
	void FixedUpdate () {

        if (!pm.canPlay)
            return;

        if (!isDead)
        {

            //Shoot();
            if (fireRate == 0)
                //{
                //Debug.Log("1");
                //single burst

                //print(isFarting);
                if (!isFarting)
                {
                    CheckDeath();
                }

            CheckEdgeTile();

            if ((player.GetButtonUp("Fire") || (nextBullet > 0 && !(player.GetButton("Fire")))) && canShot)
            {
                //print(nextBullet);

                canShot = false;
                Shoot();
                Invoke("ChangeCanShotState", nextShot);

                //print(nextBullet);

                gameObject.GetComponent<PlayerMovement>().speed = 3f;

                nextBullet = 0f;
            }



            if (player.GetButton("Fire") && canShot)
            {
                nextBullet += 0.05f;
                gameObject.GetComponent<PlayerMovement>().speed = 1.5f;

                //print(nextBullet);

                //nextBullet = 0f;
            }


            if (player.GetButtonDown("Turbo") && canFart)
            {
                //print("Fireeeeee2");
                canFart = false;

                Fart();
                Invoke("ChangeCanFartState", nextFart);
                //Shoot();
            }



            InitAnimation();
            //}
            /*
            else
            {
                Debug.Log("2");
                if (Input.GetButton("Fire1") && Time.time > timeToFire)
                {
                    timeToFire = Time.time + 1 / fireRate;
                    Shoot();
                }
            }*/
        }
    }

    
    void InitAnimation()
    {

        //if (player.GetButton("Horizontal") || player.GetButton("Vertical"))
        //{
        //    animPlayer.SetBool("Snot", false);
        //    animPlayer.SetBool("isMoving",true);
        //}

        //    if((!player.GetButton("Horizontal")) && (!player.GetButton("Vertical")))
        //    {
        //        animPlayer.SetBool("Snot", false);
        //        animPlayer.SetBool("isMoving", false);
        //    }

        //    if (Input.GetButton("Fire1"))
        //    {
        //        animPlayer.SetBool("isMoving", false);
        //        animPlayer.SetBool("Snot", true);
        //    }

        if (player.GetAxis("Move Horizontal") != 0 || player.GetAxis("Move Vertical") != 0)
        {
            animPlayer.SetBool("Snot", false);
            animPlayer.SetBool("isMoving", true);
        }

        if (player.GetAxis("Move Horizontal") == 0 && player.GetAxis("Move Vertical") == 0)
        {
            animPlayer.SetBool("Snot", false);
            animPlayer.SetBool("isMoving", false);
        }

        if (player.GetButton("Fire"))
        {
            animPlayer.SetBool("isMoving", false);
            animPlayer.SetBool("Snot", true);
        }

    }


    void ChangeCanFartState()
    {
        canFart = true;
    }

    void ChangeCanShotState()
    {
        canShot = true;
    }


    void Shoot ()
    {
        //Debug.Log("Test");
        //converte da screen coordinate a world coordinate
        Vector2 mousePosition = spawnWindPosition.position;
        //Vector2 mousePosition = new Vector2(Camera.main.ScreenToWorldPoint(Input.mousePosition).x, Camera.main.ScreenToWorldPoint(Input.mousePosition).y) ;
        Vector2 firePointPosition = new Vector2(firePoint.position.x, firePoint.position.y);

        // origin, direction, distance,layermask
        RaycastHit2D hit = Physics2D.Raycast(firePointPosition, mousePosition - firePointPosition, 100, whatToHit );


        //camShake.Shake(camShakeAmt, camShakeLenght);

        //Debug.DrawLine(firePointPosition, mousePosition);
        Debug.DrawLine(firePointPosition, (mousePosition-firePointPosition) *100, Color.cyan);


        float rotz = transform.rotation.z;
        float rotationOffset = -25;

        float speedBullet = 0f;

        if(nextBullet>= 0 && nextBullet <= 3)
        {
            speedBullet = 12f;
            //print("lento");
            source.PlayOneShot(snot);
        }

        if(nextBullet > 3 && nextBullet <= 6)
        {
            speedBullet = 16f;
            //print("medio");
            source.PlayOneShot(snot);
        }
            
        if(nextBullet >6 && nextBullet < 9)
        {
            speedBullet = 24f;
            //print("alto");
            source.PlayOneShot(superSnot);
        }



        GameObject particles = (GameObject) Resources.Load("particleSnot");
        GameObject particlesGO = Instantiate(particles);

        particlesGO.transform.SetParent(transform);
        particlesGO.transform.localPosition = new Vector3(0.2f, 0f, spawnWindPosition.localPosition.z);
        particlesGO.transform.localEulerAngles = new Vector3(0, 0, 0);


        for (int i = 0; i < 10; i++)
        {

            rotationOffset += 5;

            GameObject bullet = (GameObject)Resources.Load("Bullet");

            GameObject bulletGO = (GameObject)Instantiate(bullet);

            bulletGO.GetComponent<BulletManager>().speed = speedBullet;

            //bulletGO.transform.position = transform.position;
            //bulletGO.transform.position = new Vector3( transform.position.x, transform.position.y, transform.position.z);

            bulletGO.transform.position = new Vector3(spawnWindPosition.position.x, spawnWindPosition.position.y, spawnWindPosition.position.z);

            //bulletGO.transform.rotation = Quaternion.Euler(0f, 0f, rotz + rotationOffset);

            Vector2 direction = mousePosition - firePointPosition;

            Vector2 new_direction = direction.Rotate(rotationOffset);

            //Vector2 direction = playerShip.transform.position - bullet.transform.position;

            bulletGO.GetComponent<BulletManager>().SetDirection(new_direction);

        }

      
        if (hit.collider != null)
        {
            Debug.DrawLine(firePointPosition, hit.point, Color.red);

            /*
            Enemy enemy = hit.collider.GetComponent<Enemy>();
            if(enemy != null)
            {
                enemy.DamageEnemy(damage);
                Debug.Log(hit.collider.name + "10 damage");
            }*/

        }
        /*
        if (Time.time >= timeToSpawnEffect)
        {
            Vector3 hitPos;
            Vector3 hitNormal;

            if(hit.collider == null)
            {
                hitPos = (mousePosition - firePointPosition) * 100;
                hitNormal = new Vector3(9999, 9999, 9999);
            }
            else
            {
                hitPos = hit.point;
                hitNormal = hit.normal;
            }
            Effect(hitPos, hitNormal);
            timeToSpawnEffect = Time.time + 1 / effectSpawnRate;
        }*/

        source.PlayOneShot(windShoot);
    }





    void Fart()
    {

        source.PlayOneShot(turbo);
        source.PlayOneShot(turbo);
        source.PlayOneShot(turbo);


        isFarting = true;

        Vector2 fartPosition = new Vector2(spawnFartPosition.position.x, spawnFartPosition.position.y);
        Vector2 firePointPosition = new Vector2(firePoint.position.x, firePoint.position.y);

        // origin, direction, distance,layermask
        RaycastHit2D hit = Physics2D.Raycast(firePointPosition, fartPosition - firePointPosition, 100, whatToHit);


        //camShake.Shake(camShakeAmt, camShakeLenght);

        //Debug.DrawLine(firePointPosition, mousePosition);
        Debug.DrawLine(firePointPosition, (fartPosition - firePointPosition) * 100, Color.cyan);


        float rotz = transform.rotation.z;
        float rotationOffset = -25;

        Vector2 Fartdirection = -1*( fartPosition - firePointPosition);

        StartCoroutine( FartPush(Fartdirection));


        for (int i = 0; i < 10; i++)
        {

            rotationOffset += 5;

            GameObject bullet = (GameObject)Resources.Load("Bullet");

            GameObject bulletGO = (GameObject)Instantiate(bullet);

            bulletGO.GetComponent<BulletManager>().speed = 8f;

            //bulletGO.transform.position = transform.position;
            //bulletGO.transform.position = new Vector3( transform.position.x, transform.position.y, transform.position.z);

            bulletGO.transform.position = new Vector3(spawnFartPosition.position.x, spawnFartPosition.position.y, spawnWindPosition.position.z);

            //bulletGO.transform.rotation = Quaternion.Euler(0f, 0f, rotz + rotationOffset);

            Vector2 direction = fartPosition - firePointPosition;

            Vector2 new_direction = direction.Rotate(rotationOffset);

            //Vector2 direction = playerShip.transform.position - bullet.transform.position;

            bulletGO.GetComponent<BulletManager>().SetDirection(new_direction);

        }

    }



    IEnumerator  FartPush(Vector2 direction)
    {
        //print("Entrato in FartPush");
        float time = 0;
        Vector2 position = transform.position;
        float newSpeed = speedFart - Mathf.Pow(time, 4);

        //print("FAAAART");
        //transform.GetComponent<Rigidbody2D>().AddForce(direction * 10000);
        //yield return 0;

        newSpeed = speedFart;
        
        while (newSpeed >= 0) {
            //print("Entrato in while");

            newSpeed -= time;

            //newSpeed = speedFart - Mathf.Pow(time, 4);

            //print(newSpeed);

            transform.GetComponent<Rigidbody2D>().AddForce(direction * newSpeed);

            //position += direction * (newSpeed) * Time.deltaTime;
            //transform.position = position;

            time += 5f;
            //time += 5f;
            //time += 0.05f;
             yield return 0.1f;
        }

        if (newSpeed < 0)
        {
            isFarting = false;
        }
    }









    void Effect (Vector3 hitPos, Vector3 hitNormal)
    {
        var position = firePoint.position;
        var rotation = firePoint.rotation;

        Transform trail = (Transform) Instantiate(BulletTrailPrefab, position, rotation) ;
        LineRenderer lr = trail.GetComponent<LineRenderer>();

        if(lr!= null)
        {
            //prende l'indice in prima
            lr.SetPosition(0, position);
            lr.SetPosition(1, hitPos);
        }

       // lr.gameObject.SetActive(false);
        //Destroy(trail.gameObject, 1f);

        if(hitNormal != new Vector3(9999, 9999, 9999))
        {
            Transform hitParticle = (Transform)Instantiate(HitPrefab, hitPos, Quaternion.FromToRotation(Vector3.right, hitNormal));
            Destroy(hitParticle.gameObject, 1f);
        }
        
        //Transform clone =(Transform)Instantiate(MuzzleFlashPrefab, position, rotation);
        //clone.parent = firePoint;
        //float size = Random.Range(0.6f, 0.9f);
        //clone.localScale = new Vector3(size, size, size);
        //Destroy(clone.gameObject, 0.03f);

        camShake.Shake(camShakeAmt, camShakeLenght);
    }







    void OnCollisionEnter2D(Collision2D col)
    {

        //print("COLLISION"+ col.gameObject);

        
        if (col.gameObject.tag == "Obstacle")
        {
            //print("Obstacleeee");
            //print("PRESOOOOOO");

            if (isFarting)
            {
                foreach (ContactPoint2D contact in col.contacts)
                {

                    Vector2 movem = new Vector2(GetComponent<PlayerMovement>().movement.x, GetComponent<PlayerMovement>().movement.y);
                    //Debug.DrawRay(contact.point, contact.normal, Color.white);
                    Vector2 reflectDir = Vector2.Reflect(movem, contact.normal);

                    gameObject.GetComponent<Rigidbody2D>().AddForce(reflectDir * 300);
                    //transform.rotation.SetFromToRotation(_direction, reflectDir);
                    //return;
                }
            }

            //Destroy(gameObject);

        }


        /*

        if (col.gameObject.tag == "PhysicsGO")
        {
            print("AddForce");
            print(col);
            col.gameObject.GetComponent<Rigidbody2D>().AddForce(_direction * 100);
            Destroy(gameObject);
        } */


        if (col.gameObject.tag == "Player" || col.gameObject.tag == "PhysicsGO")
        {
            //print("SPIIIINGOOOO");
            //print("AddForce");
            //print(col);
            if (isFarting)
            {
                Vector2 movem = new Vector2(GetComponent<PlayerMovement>().movement.x, GetComponent<PlayerMovement>().movement.y);

                col.gameObject.GetComponent<Rigidbody2D>().AddForce(movem * 800);
                //Destroy(gameObject);
            }
        }
        


    
    }




}
