﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class WindManager : MonoBehaviour {

	[HideInInspector] public static WindManager Instance;
	private static bool _istantiated = false;

	float windDirection = 45f;
	float windForce = 0.07f;
	float windSpeed = 2f;

	[HideInInspector] public float tempWindDirection;
	[HideInInspector] public float tempWindForce;
	[HideInInspector] public float tempWindSpeed;

	[HideInInspector] public float finalWindSpeed;
	[HideInInspector] public Vector2 windVector;
	[HideInInspector] public ArrayList grassWinds = new ArrayList();
	[HideInInspector] public ArrayList physicsObjects = new ArrayList();
	[HideInInspector] public ArrayList fogObjects = new ArrayList();
	[HideInInspector] public ArrayList fogSprites = new ArrayList();

	Sprite fogSprite;
	bool fogEnabled;
	float fogAlpha;
	float speedVar;

	void Awake () {

		if (_istantiated){
			Destroy(this);
			return;
		}
		_istantiated = true;
		Instance = this;

		DontDestroyOnLoad(gameObject);
		Application.targetFrameRate = 60;
		Time.fixedDeltaTime = 1f/60f;
		Screen.sleepTimeout = 0;

		fogSprite = Resources.Load<Sprite>("Sprites/Particles/particle_cloud");
		DisableWindAndRestart();
		Invoke("EnableFog",Random.Range(6f,10f));
	}

	void Start () {

		Transform tiles = GameObject.Find("Tiles").transform;
		foreach(Transform t in tiles){
			if (t.name.ToLower().Contains("grass")){
				if (t.GetChild(0).GetComponent<GrassWind>())
					grassWinds.Add(t.GetChild(0).GetComponent<GrassWind>());
			}
		}
	}

	void FixedUpdate () {
		
		tempWindDirection = Mathf.SmoothStep(tempWindDirection, windDirection, 0.1f);
		tempWindForce = Mathf.SmoothStep(tempWindForce, windForce, 0.1f);
		tempWindSpeed = Mathf.SmoothStep(tempWindSpeed, windSpeed, 0.1f);

		speedVar += 1f;
		speedVar = Mathf.Repeat(speedVar,360f);
		finalWindSpeed = tempWindSpeed + (Mathf.Sin(speedVar * Mathf.Deg2Rad) * (tempWindSpeed/3f));
		
		windVector = new Vector2(Mathf.Sin(Mathf.Deg2Rad * tempWindDirection), Mathf.Cos(Mathf.Deg2Rad * tempWindDirection));

		//Se il vento soffia, cicla tutti gli oggetti nell'arraylist di oggetti mobili e spostali
		if (tempWindForce > 0){
			foreach (Rigidbody2D rb in physicsObjects){

				try{
					rb.AddForce(windVector * tempWindForce * 10f);
				}
				catch{}
			}

			foreach (Rigidbody2D rb in fogObjects){

				try{
					rb.AddForce(windVector * tempWindForce * 10f);
				}
				catch{}
			}
		}

		if (fogEnabled && fogAlpha < 1)
			fogAlpha += 0.003f;
		else if (!fogEnabled && fogAlpha > 0)
			fogAlpha -= 0.003f;
		foreach (SpriteRenderer fogSprite in fogSprites)
			fogSprite.color = new Color(1,1,1,fogAlpha*0.6f);

		if (fogAlpha <= 0 && !fogEnabled && fogObjects.Count > 0){
			foreach (Rigidbody2D rb in fogObjects){
				try{
					Destroy(rb.gameObject);
				}
				catch{}
			}
			fogSprites.Clear();
		}
	}

	void EnableWind () {

		windDirection = Random.Range(0f,360f);
		windForce = Random.Range(0.03f,0.08f);
		windSpeed = windForce * 35;

		if (Random.Range(0,3) == 0)
			Invoke("EnableWind",Random.Range(6f,9f));
		else
			Invoke("DisableWindAndRestart",Random.Range(6f,9f));
	}

	void DisableWindAndRestart () {

		windForce = 0f;
		windSpeed = 0f;
		Invoke("EnableWind",Random.Range(5f,10f));
	}

	void EnableFog () {

		fogAlpha = 0;
		fogEnabled = true;
		Vector2 randomSpawnPoint = new Vector2(Random.Range(-15f,15f), Random.Range(-10f,10f));
		for (int i = 0; i < 30; i++){
			
			Vector2 cloudPoint = CalculatePoint(8);
			InstantiateFogElement (randomSpawnPoint + cloudPoint);
		}
		Invoke("DisableFogAndRestart",Random.Range(10f,14f));
	}

	void DisableFogAndRestart () {

		fogEnabled = false;
		Invoke("EnableFog",Random.Range(10f,15f));
	}

	void InstantiateFogElement (Vector2 position) {

		GameObject fog = new GameObject();
		fog.name = "Fog";
		fog.transform.position = position;
		fog.transform.localScale = Vector3.one * Random.Range(1.7f,2.2f);
		if (Random.Range(0f,1f) < 0.5f)
			fog.transform.localScale = new Vector3(-fog.transform.localScale.x, fog.transform.localScale.y, fog.transform.localScale.z);

		SpriteRenderer sr = fog.AddComponent<SpriteRenderer>();
		sr.sprite = fogSprite;
		sr.color = new Color(1,1,1,fogAlpha);
		sr.sortingOrder = 8;
        sr.sortingLayerName = "Cloud";

		Rigidbody2D rb = fog.AddComponent<Rigidbody2D>();
		rb.gravityScale = 0f;
		rb.drag = fog.transform.localScale.y / 5f;

		fogSprites.Add(sr);
		fogObjects.Add(rb);
	}

	private Vector2 CalculatePoint (float r) {
		
		float angle = Random.Range(0f,1f) * Mathf.PI * 2;
		float radius = Mathf.Sqrt(Random.Range(0f,1f)) * r;
		float x = radius * Mathf.Cos(angle);
		float y = radius * Mathf.Sin(angle);
		return new Vector2(x,y);
	}
}
